# Copyright 2014 iWeb Technologies Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import webob.dec

import wafflehaus.base
import wafflehaus.resource_filter as rf


class VisibleFilter(wafflehaus.base.WafflehausBase):

    def __init__(self, app, conf):
        super(VisibleFilter, self).__init__(app, conf)
        self.log.name = conf.get('log_name', __name__)
        self.log.info('Starting wafflehaus visible image filter middleware')
        self.visible_metadata = conf.get('visible_metadata', 'visible')
        self.roles_whitelist = conf.get('roles_whitelist', 'admin')
        self.roles_whitelist = [r.strip()
                                for r in self.roles_whitelist.split()]
        self.resource = conf.get('resource',
                                 'GET /images, GET /images/detail, '
                                 'GET /v1/images, GET /v1/images/detail, '
                                 'GET /v2/images, GET /v2/images/detail')
        self.resources = rf.parse_resources(self.resource)

        if not self.roles_whitelist:
            self.log.debug("No whitelisted roles.")

    def _is_visible(self, image):
        return image.get('properties', {}).get(
            self.visible_metadata, "1") != "0"

    def _filter_non_visible_images(self, req):
        self.log.debug('Filtering non-visible images')
        response = req.get_response(self.app)
        body = response.json
        images = body.get('images')

        body['images'] = [i for i in images if self._is_visible(i)]
        response.json = body

        return response

    def _is_whitelisted(self, req):
        """Return True if role is whitelisted or roles cannot be determined."""

        if not self.roles_whitelist:
            return False

        if not hasattr(req, 'context'):
            self.log.info("No context found.")
            return False

        if not hasattr(req.context, 'roles'):
            self.log.info("No roles found in context")
            return False

        roles = req.context.roles
        self.log.debug("Received request from user with roles: %s",
                       ' '.join(roles))
        for key in self.roles_whitelist:
            if key in roles:
                self.log.debug("User role (%s) is whitelisted.", key)
                return True

        return False

    @webob.dec.wsgify
    def __call__(self, req):
        if not self.enabled:
            return self.app

        if not rf.matched_request(req, self.resources):
            self.log.debug("Request not matching resource filters (skipping)")
            return self.app

        if self._is_whitelisted(req):
            return self.app

        return self._filter_non_visible_images(req)


def filter_factory(global_conf, **local_conf):
    """Returns a WSGI filter app for use with paste.deploy."""
    conf = global_conf.copy()
    conf.update(local_conf)

    def visible(app):
        return VisibleFilter(app, conf)
    return visible



# Copyright 2014 iWeb Technologies Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json

import webob.dec
import webob.exc

import wafflehaus.base
import wafflehaus.resource_filter as rf


class BlacklistFilter(wafflehaus.base.WafflehausBase):

    def __init__(self, app, conf):
        super(BlacklistFilter, self).__init__(app, conf)
        self.log.name = conf.get('log_name', __name__)
        self.log.info('Starting wafflehaus blacklist user filter middleware')
        self.blacklist = conf.get('blacklist', '')
        self.blacklist = [u.strip() for u in self.blacklist.split()]
        self.resource = conf.get('resource',
                                 'POST /v2.0/tokens, POST /v3/auth/tokens')
        self.resources = rf.parse_resources(self.resource)

    def _filter_blacklisted_users(self, req):

        try:
            payload = json.loads(req.body)
        except ValueError:
            self.log.warning('Failed to parse json payload')
            return self.app

        auth = payload.get('auth', {})
        username = None

        # Identity v2.0
        if 'passwordCredentials' in auth:
            self.log.debug('Authenticating with Identity v2.0')
            username = auth.get('passwordCredentials').get('username')
        # Identity v3
        elif 'identity' in auth:
            self.log.debug('Authenticating with Identity v3')
            # Password method
            password_method = auth.get('identity', {}).get('password')
            if password_method:
                self.log.debug('Authenticating with password method')
                username = password_method.get('user', {}).get('name')
            else:
                self.log.warning('Unknown authentication method')
        else:
            self.log.warning('Failed to parse auth credentials')
            return self.app

        self.log.debug('User %s is authenticating', username)
        if username in self.blacklist:
            self.log.info('User %s is blacklisted', username)
            return webob.exc.HTTPForbidden()

        self.log.debug('User %s is not blacklisted', username)
        return self.app

    @webob.dec.wsgify
    def __call__(self, req):
        if not self.enabled:
            return self.app

        if not self.blacklist:
            self.log.debug('No blacklisted users (skipping)')
            return self.app

        if not rf.matched_request(req, self.resources):
            self.log.debug('Request not matching resource filters (skipping)')
            return self.app

        return self._filter_blacklisted_users(req)


def filter_factory(global_conf, **local_conf):
    """Returns a WSGI filter app for use with paste.deploy."""
    conf = global_conf.copy()
    conf.update(local_conf)

    def blacklist(app):
        return BlacklistFilter(app, conf)
    return blacklist



import json

import webob.dec
import webob.request
import webob.response

from wafflehaus.iweb.glance.image_filter import visible
from wafflehaus import tests


class FakeWebApp(object):
    def __init__(self, response=None):
        self.set_response(response)

    def set_response(self, response):
        self.response = response
        self.body = response.body

    @webob.dec.wsgify
    def __call__(self, req):
        return self.response


class RequestContext(object):
    pass


class TestImageFilter(tests.TestCase):

    def setUp(self):
        self.conf1 = {'enabled': 'true',
                      'visible_metadata': 'is_visible'}
        self.conf2 = {'enabled': 'false',
                      'visible_metadata': 'is_visible'}
        self.conf3 = {'enabled': 'true',
                      'visible_metadata': 'is_visible',
                      'roles_whitelist': ''}
        self.body1 = json.dumps({'images': [
            {"id": "0e4cf274-5637-4412-957c-a1584743f41c",
             "name": "Image A",
             "properties": {"version": "1", "is_visible": "0"}},
            {"id": "517f7f5f-37b5-407e-a36d-c9163d606934",
             "name": "Image A",
             "properties": {"version": "2", "is_visible": "1"}},
            {"id": "51a553e6-0b9a-4cff-8e69-56363271575c",
             "name": "Image B",
             "properties": {"version": "1"}},
            {"id": "1fa0356c-73a2-4306-94b9-26c04f49e9f2",
             "name": "Image C",
             "properties": {"is_visible": "invalid"}},
            {"id": "cdbc922d-eabc-4b67-b21e-f88b6dd37c4b",
             "name": "Image D",
             "properties": {}},
        ]})
        self.FILTERED_IMAGES_COUNT = 4
        self.ALL_IMAGES_COUNT = 5
        self.app = FakeWebApp(response=self.create_response(self.body1))

    def create_filter(self, conf):
        return visible.filter_factory(conf)(self.app)

    def create_context(self, **kwargs):
        ctxt = RequestContext()
        for key in kwargs:
            setattr(ctxt, key, kwargs[key])
        return ctxt

    def create_request(self, url, method, context=None):
        req = webob.request.Request.blank(url, method=method)
        if context:
            req.context = context
        return req

    def create_response(self, body):
        return webob.response.Response(body=body)

    def assertImageCount(self, count, resp):
        body_images = json.loads(resp.body).get("images")
        self.assertEqual(count, len(body_images))

    def test_visible_disabled(self):
        """Images are not filtered when disabled."""
        filter = self.create_filter(self.conf2)

        req = self.create_request('/v1/images', method='GET')
        resp = filter.__call__(req)

        self.assertImageCount(self.ALL_IMAGES_COUNT, resp)

    def test_visible_context_not_found(self):
        """Images are filtered if no context is found."""
        filter = self.create_filter(self.conf1)

        req = self.create_request('/v1/images', method='GET')
        resp = filter.__call__(req)

        self.assertTrue(isinstance(resp, webob.response.Response))
        self.assertNotEqual(self.app.body, resp.body)
        self.assertImageCount(self.FILTERED_IMAGES_COUNT, resp)

    def test_visible_no_roles_whitelist(self):
        """Images are filtered if no roles are whitelisted."""
        filter = self.create_filter(self.conf3)

        ctxt = self.create_context(roles=["_member_"])
        req = self.create_request('/v1/images', method='GET', context=ctxt)
        resp = filter.__call__(req)

        self.assertTrue(isinstance(resp, webob.response.Response))
        self.assertNotEqual(self.app.body, resp.body)
        self.assertImageCount(self.FILTERED_IMAGES_COUNT, resp)

    def test_visible_roles_not_found(self):
        """Images are filtered if no context is found."""
        filter = self.create_filter(self.conf1)

        ctxt = self.create_context()
        req = self.create_request('/v1/images', method='GET', context=ctxt)
        resp = filter.__call__(req)

        self.assertTrue(isinstance(resp, webob.response.Response))
        self.assertNotEqual(self.app.body, resp.body)
        self.assertImageCount(self.FILTERED_IMAGES_COUNT, resp)

    def test_visible_images_are_filtered_v1(self):
        """Images are filtered (Glance v1)."""
        filter = self.create_filter(self.conf1)

        ctxt = self.create_context(roles=["_member_"])
        req = self.create_request('/v1/images', method='GET', context=ctxt)
        resp = filter.__call__(req)

        self.assertTrue(isinstance(resp, webob.response.Response))
        self.assertNotEqual(self.app.body, resp.body)
        self.assertImageCount(self.FILTERED_IMAGES_COUNT, resp)

    def test_visible_images_are_filtered_v2(self):
        """Images are filtered (Glance v2)."""
        filter = self.create_filter(self.conf1)

        ctxt = self.create_context(roles=["_member_"])
        req = self.create_request('/v2/images', method='GET', context=ctxt)
        resp = filter.__call__(req)

        self.assertTrue(isinstance(resp, webob.response.Response))
        self.assertNotEqual(self.app.body, resp.body)
        self.assertImageCount(self.FILTERED_IMAGES_COUNT, resp)

    def test_visible_roles_whitelist_v1(self):
        """Images are not filtered for whitelisted roles (Glance v1)."""
        filter = self.create_filter(self.conf1)

        ctxt = self.create_context(roles=["admin"])
        req = self.create_request('/v1/images', method='GET', context=ctxt)
        resp = filter.__call__(req)

        self.assertImageCount(self.ALL_IMAGES_COUNT, resp)

    def test_visible_roles_whitelist_v2(self):
        """Images are not filtered for whitelisted roles (Glance v2)."""
        filter = self.create_filter(self.conf1)

        ctxt = self.create_context(roles=["admin"])
        req = self.create_request('/v2/images', method='GET', context=ctxt)
        resp = filter.__call__(req)

        self.assertImageCount(self.ALL_IMAGES_COUNT, resp)



import json
import mock

import webob.exc

from wafflehaus.iweb.keystone.user_filter import blacklist
from wafflehaus import tests


class TestUserFilter(tests.TestCase):

    def setUp(self):
        self.app = mock.Mock()
        self.conf1 = {'enabled': 'true', 'blacklist': 'admin nova'}
        self.body_v2 = json.dumps({'auth': {
            "tenantName": "demo",
            "passwordCredentials": {"username": "%(username)s",
                                    "password": "s3cr3t"}}})
        self.body_v3 = json.dumps({"auth": {
            "scope": {
                "project": {
                    "domain": {"id": "default"},
                    "name": "demo"}},
            "identity": {
                "password": {
                    "user": {
                        "domain": {"id": "default"},
                        "password": "s3cr3t",
                        "name": "%(username)s"}},
            "methods": ["password"]}}})

    def create_filter(self, conf):
        return blacklist.filter_factory(conf)(self.app)

    def create_request(self, url, body):
        req = webob.request.Request.blank(url, method='POST')
        req.body = body
        return req

    def test_blacklist_deny_blacklisted_user_v2(self):
        filter = self.create_filter(self.conf1)

        body = self.body_v2 % {'username': 'admin'}
        req = self.create_request('/v2.0/tokens', body=body)
        resp = filter.__call__(req)

        self.assertTrue(isinstance(resp, webob.exc.HTTPException))

    def test_blacklist_deny_blacklisted_user_v3(self):
        filter = self.create_filter(self.conf1)

        body = self.body_v3 % {'username': 'admin'}
        req = self.create_request('/v3/auth/tokens', body=body)
        resp = filter.__call__(req)

        self.assertTrue(isinstance(resp, webob.exc.HTTPException))

    def test_blacklist_allow_not_blacklisted_user_v2(self):
        filter = self.create_filter(self.conf1)

        body = self.body_v2 % {'username': 'demo'}
        req = self.create_request('/v2.0/tokens', body=body)
        resp = filter.__call__(req)

        self.assertEqual(self.app, resp)

    def test_blacklist_allow_not_blacklisted_user_v3(self):
        filter = self.create_filter(self.conf1)

        body = self.body_v3 % {'username': 'demo'}
        req = self.create_request('/v3/auth/tokens', body=body)
        resp = filter.__call__(req)

        self.assertEqual(self.app, resp)

    def test_blacklist_disabled_v2(self):
        conf = {'enabled': 'false', 'blacklist': 'admin nova'}
        filter = self.create_filter(conf)

        body = self.body_v2 % {'username': 'admin'}
        req = self.create_request('/v2.0/tokens', body=body)
        resp = filter.__call__(req)

        self.assertEqual(self.app, resp)

    def test_blacklist_no_blacklist_v2(self):
        conf = {'enabled': 'true', 'blacklist': ''}
        filter = self.create_filter(conf)

        body = self.body_v2 % {'username': 'admin'}
        req = self.create_request('/v2.0/tokens', body=body)
        resp = filter.__call__(req)

        self.assertEqual(self.app, resp)

    def test_blacklist_unknown_auth(self):
        filter = self.create_filter(self.conf1)

        body = json.dumps({'auth': {}})
        req = self.create_request('/v2.0/tokens', body=body)
        resp = filter.__call__(req)

        self.assertEqual(self.app, resp)

    def test_blacklist_unknown_auth_v3(self):
        filter = self.create_filter(self.conf1)

        body = json.dumps({'auth': {'identity': {}}})
        req = self.create_request('/v3/auth/tokens', body=body)
        resp = filter.__call__(req)

        self.assertEqual(self.app, resp)

    def test_blacklist_junk_body(self):
        """Ignore junk body."""
        filter = self.create_filter(self.conf1)

        req = self.create_request('/v2.0/tokens', body='junk')
        resp = filter.__call__(req)

        self.assertEqual(self.app, resp)
